@@ -1,6 +1,7 @@
 # encoding: utf-8
 """
-Copyright 2011 iACT, Universite de Montreal, Jean Piche, Olivier Belanger, Jean-Michel Dumas
+Copyright 2019 iACT, Universite de Montreal,
+Jean Piche, Olivier Belanger, Jean-Michel Dumas
 
 This file is part of Cecilia 5.
 
@@ -18,14 +19,14 @@ You should have received a copy of the GNU General Public License
 along with Cecilia 5.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-import os, sys, wx, time, math, copy, codecs
-import pprint as pp
+import os, sys, wx, time, math, copy, codecs, shutil
 import unicodedata
 from subprocess import Popen
 from .constants import *
 from .API_interface import *
 import Resources.Variables as vars
 import wx.lib.agw.supertooltip as STT
+import xmlrpc.client as xmlrpclib
 
 if sys.version_info[0] < 3:
     unicode_t = unicode
@@ -233,11 +234,11 @@ def stopCeciliaSound():
             setVar('spectrumFrame', None)
     if getVar("audioServer").isAudioServerRunning():
         getVar("audioServer").stop()
+        time.sleep(.25)
         if getVar("currentModule") is not None:
             getVar("audioServer").checkForAutomation()
             getVar("currentModule")._checkForAutomation()
             getVar("grapher").checkForAutomation()
-        time.sleep(.25)
     resetControls()
 
 def resetControls():
@@ -251,7 +252,8 @@ def resetControls():
         wx.CallAfter(getControlPanel().vuMeter.reset)
 
 def queryAudioMidiDrivers():
-    inputs, inputIndexes, defaultInput, outputs, outputIndexes, defaultOutput, midiInputs, midiInputIndexes, defaultMidiInput = getVar("audioServer").getAvailableAudioMidiDrivers()
+    inputs, inputIndexes, defaultInput, outputs, outputIndexes, \
+    defaultOutput, midiInputs, midiInputIndexes, defaultMidiInput = getVar("audioServer").getAvailableAudioMidiDrivers()
     setVar("availableAudioOutputs", outputs)
     setVar("availableAudioOutputIndexes", outputIndexes)
     if getVar("audioOutput") not in outputIndexes:
@@ -281,6 +283,7 @@ def queryAudioMidiDrivers():
             setVar("midiDeviceIn", 0)
 
 def openAudioFileDialog(parent, wildcard, type='open', defaultPath=os.path.expanduser('~')):
+    setVar("canGrabFocus", False)
     openDialog = wx.FileDialog(parent, message='Choose a file to %s' % type,
                                 defaultDir=defaultPath, wildcard=wildcard,
                                 style=wx.FD_OPEN | wx.FD_PREVIEW)
@@ -290,6 +293,7 @@ def openAudioFileDialog(parent, wildcard, type='open', defaultPath=os.path.expan
     else:
         filePath = None
     openDialog.Destroy()
+    setVar("canGrabFocus", True)
     return filePath
 
 def saveFileDialog(parent, wildcard, type='Save'):
@@ -300,6 +304,7 @@ def saveFileDialog(parent, wildcard, type='Save'):
         defaultPath = getVar("saveFilePath", unicode=True)
         ext = ".c5"
 
+    setVar("canGrabFocus", False)
     defaultFile = os.path.split(getVar("currentCeciliaFile", unicode=True))[1].split(".")[0]
     saveAsDialog = wx.FileDialog(parent, message="%s file as ..." % type,
                                  defaultDir=defaultPath, defaultFile=defaultFile + ext,
@@ -313,38 +318,18 @@ def saveFileDialog(parent, wildcard, type='Save'):
     else:
         filePath = None
     saveAsDialog.Destroy()
+    setVar("canGrabFocus", True)
     return filePath
 
-def saveBeforeClose(parent):
-    if getVar("isModified"):
-        saveBeforeCloseDialog = wx.MessageDialog(parent,
-                        'This file has been modified since the last save point. \
-                        Would you like to save the changes?',
-                        'Save Changes?', wx.YES_NO | wx.CANCEL | wx.YES_DEFAULT)
-    else:
-        return True
-
-    answer = saveBeforeCloseDialog.ShowModal()
-    if answer == wx.ID_YES:
-        if saveCeciliaFile(parent, False):
-            result = True
-        else:
-            result = False
-    elif answer == wx.ID_NO:
-        result = True
-    elif answer == wx.ID_CANCEL:
-        result = False
-
-    saveBeforeCloseDialog.Destroy()
-    return result
-
 def showErrorDialog(title, msg):
+    setVar("canGrabFocus", False)
     if getVar("mainFrame") is not None:
         dlg = wx.MessageDialog(getVar("mainFrame"), msg, title, wx.OK)
     else:
         dlg = wx.MessageDialog(None, msg, title, wx.OK)
     dlg.ShowModal()
     dlg.Destroy()
+    setVar("canGrabFocus", True)
 
 ###### External app calls ######
 def loadPlayerEditor(app_type):
@@ -357,6 +342,7 @@ def loadPlayerEditor(app_type):
     else:
         wildcard = "All files|*"
 
+    setVar("canGrabFocus", False)
     path = ''
     dlg = wx.FileDialog(None, message="Choose a %s..." % app_type,
                         defaultDir=ensureNFD(os.path.expanduser('~')),
@@ -365,6 +351,7 @@ def loadPlayerEditor(app_type):
     if dlg.ShowModal() == wx.ID_OK:
         path = dlg.GetPath()
     dlg.Destroy()
+    setVar("canGrabFocus", True)
 
     if path:
         if app_type == 'soundfile player':
@@ -438,17 +425,25 @@ def openCurrentFileAsText(curfile):
                 print('Unable to open desired software:\n' + app)
 
 ###### Preset functions ######
-def deletePreset(preset):
-    del vars.CeciliaVar['presets'][preset]
+def loadPresetFromFile(preset):
+    presetFile = os.path.join(PRESETS_PATH, getVar("currentModuleName"), preset)
 
-def loadPresetFromDict(preset):
-    currentModule = getVar("currentModule")
-    setVar("currentModule", None)
-    if preset in getVar("presets") or preset == "init":
-        if preset == "init":
-            presetData = getVar("initPreset")
-        else:
-            presetData = getVar("presets")[preset]
+    presetData = None
+    if preset == "init":
+        presetData = getVar("initPreset")
+    elif os.path.isfile(presetFile):
+        with open(presetFile, 'r') as f:
+            try:
+                result, method = xmlrpclib.loads(f.read())
+                presetData = result[0]
+            except:
+                showErrorDialog("Preset corrupted...", "Failed to load preset '%s', reloading 'init'..." % preset)
+                preset = "init"
+                presetData = getVar("initPreset")
+
+    if presetData is not None:
+        currentModule = getVar("currentModule")
+        setVar("currentModule", None)
 
         for data in presetData.keys():
             if data == 'userInputs':
@@ -484,13 +479,13 @@ def loadPresetFromDict(preset):
                         slider.setState(slidersDict[slider.getName()])
                 del slidersDict
             elif data == 'plugins':
-                pluginsDict = copy.deepcopy(presetData[data])
+                pluginsDict = deepCopy(presetData[data])
                 wx.CallAfter(getControlPanel().setPlugins, pluginsDict)
                 del pluginsDict
             elif data == 'userTogglePopups':
                 togDict = presetData[data]
                 for widget in getVar("userTogglePopups"):
-                    if widget.getName() in togDict:
+                    if widget.getName() in togDict and hasattr(widget, "setValue"):
                         widget.setValue(togDict[widget.getName()], True)
                 del togDict
             else:
@@ -502,17 +497,17 @@ def loadPresetFromDict(preset):
                 except:
                     pass
         elif 'userGraph' in presetData:
-            graphDict = presetData['userGraph']
+            graphDict = deepCopy(presetData['userGraph'])
             ends = ['min', 'max']
             for line in graphDict:
                 for i, graphLine in enumerate(getVar("grapher").getPlotter().getData()):
                     if line == graphLine.getName():
-                        graphLine.setLineState(copy.deepcopy(graphDict[line]))
+                        graphLine.setLineState(graphDict[line])
                         break
                     else:
                         for end in ends:
                             if graphLine.getLabel().endswith(end) and line.endswith(end) and line.startswith(graphLine.getName()):
-                                graphLine.setLineState(copy.deepcopy(graphDict[line]))
+                                graphLine.setLineState(graphDict[line])
                                 break
             del graphDict
 
@@ -536,13 +531,13 @@ def againForPluginKnobs(presetData):
         for line in graphDict:
             for i, graphLine in enumerate(getVar("grapher").getPlotter().getData()):
                 if line == graphLine.getName():
-                    graphLine.setLineState(copy.deepcopy(graphDict[line]))
+                    graphLine.setLineState(graphDict[line])
                     break
         del graphDict
         getVar("grapher").getPlotter().draw()
         getVar("grapher").setTotalTime(getVar("totalTime"))
 
-def savePresetToDict(presetName):
+def savePresetToFile(presetName):
     presetDict = dict()
     presetDict['nchnls'] = getVar("nchnls")
     presetDict['totalTime'] = getVar("totalTime")
@@ -560,9 +555,9 @@ def savePresetToDict(presetName):
         plugins = getVar("plugins")
         for i, plugin in enumerate(plugins):
             if plugin is None:
-                widgetDict[i] = ['None', [0, 0, 0, 0], [[0, 0, None], [0, 0, None], [0, 0, None]]]
+                widgetDict[str(i)] = ['None', [0, 0, 0, 0], [[0, 0, None], [0, 0, None], [0, 0, None]]]
             else:
-                widgetDict[i] = [plugin.getName(), plugin.getParams(), plugin.getStates()]
+                widgetDict[str(i)] = [plugin.getName(), plugin.getParams(), plugin.getStates()]
         presetDict['plugins'] = copy.deepcopy(widgetDict)
         del widgetDict
 
@@ -594,28 +589,32 @@ def savePresetToDict(presetName):
         presetDict['userGraph'] = copy.deepcopy(graphDict)
         del graphDict
 
-    if presetName == "init":
-        setVar("initPreset", copy.deepcopy(presetDict))
-    else:
-        getVar("presets")[presetName] = copy.deepcopy(presetDict)
-        setVar("isModified", True)
+        if presetName == "init":
+            setVar("initPreset", deepCopy(presetDict))
+        else:
+            with open(os.path.join(PRESETS_PATH, getVar('currentModuleName'), presetName), "w") as presetFile:        
+                msg = xmlrpclib.dumps((presetDict, ), allow_none=True)
+                presetFile.write(msg)
 
 def completeUserInputsDict():
     for i in getVar("userInputs"):
-        getVar("userInputs")[i]['mode'] = 0
-        if getVar("userInputs")[i]['type'] == 'csampler':
-            cfilein = getControlPanel().getCfileinFromName(i)
-            getVar("userInputs")[i]['off' + cfilein.getName()] = cfilein.getOffset()
-            getVar("userInputs")[i]['loopMode'] = cfilein.getSamplerInfo()['loopMode']
-            getVar("userInputs")[i]['startFromLoop'] = cfilein.getSamplerInfo()['startFromLoop']
-            getVar("userInputs")[i]['loopX'] = cfilein.getSamplerInfo()['loopX']
-            getVar("userInputs")[i]['loopIn'] = cfilein.getSamplerInfo()['loopIn']
-            getVar("userInputs")[i]['loopOut'] = cfilein.getSamplerInfo()['loopOut']
-            getVar("userInputs")[i]['gain'] = cfilein.getSamplerInfo()['gain']
-            getVar("userInputs")[i]['transp'] = cfilein.getSamplerInfo()['transp']
-        elif getVar("userInputs")[i]['type'] == 'cfilein':
-            cfilein = getControlPanel().getCfileinFromName(i)
-            getVar("userInputs")[i]['off' + cfilein.getName()] = cfilein.getOffset()
+        try:
+            getVar("userInputs")[i]['mode'] = 0
+            if getVar("userInputs")[i]['type'] == 'csampler':
+                cfilein = getControlPanel().getCfileinFromName(i)
+                getVar("userInputs")[i]['off' + cfilein.getName()] = cfilein.getOffset()
+                getVar("userInputs")[i]['loopMode'] = cfilein.getSamplerInfo()['loopMode']
+                getVar("userInputs")[i]['startFromLoop'] = cfilein.getSamplerInfo()['startFromLoop']
+                getVar("userInputs")[i]['loopX'] = cfilein.getSamplerInfo()['loopX']
+                getVar("userInputs")[i]['loopIn'] = cfilein.getSamplerInfo()['loopIn']
+                getVar("userInputs")[i]['loopOut'] = cfilein.getSamplerInfo()['loopOut']
+                getVar("userInputs")[i]['gain'] = cfilein.getSamplerInfo()['gain']
+                getVar("userInputs")[i]['transp'] = cfilein.getSamplerInfo()['transp']
+            elif getVar("userInputs")[i]['type'] == 'cfilein':
+                cfilein = getControlPanel().getCfileinFromName(i)
+                getVar("userInputs")[i]['off' + cfilein.getName()] = cfilein.getOffset()
+        except:
+            pass
     return copy.deepcopy(getVar("userInputs"))
 
 def updateInputsFromDict():
@@ -657,57 +656,57 @@ def saveRuntimeBackupFile(cecFilePath):
     with open(MODULE_RUNTIME_BACKUP_PATH, "w") as f:
         f.write(_module)
 
-def saveCeciliaFile(parent, showDialog=True):
-    if getVar("builtinModule") or showDialog:
-        wildcard = "Cecilia file (*.%s)|*.%s" % (FILE_EXTENSION, FILE_EXTENSION)
-        fileToSave = saveFileDialog(parent, wildcard, 'Save')
-        if not fileToSave:
-            return False
-        else:
-            if not fileToSave.endswith(FILE_EXTENSION):
-                fileToSave = "%s.%s" % (fileToSave, FILE_EXTENSION)
+def saveCeciliaFile(parent):
+    wildcard = "Cecilia file (*.%s)|*.%s" % (FILE_EXTENSION, FILE_EXTENSION)
+    fileToSave = saveFileDialog(parent, wildcard, 'Save')
+    if not fileToSave:
+        return
     else:
-        fileToSave = getVar("currentCeciliaFile", unicode=True)
+        if not fileToSave.endswith(FILE_EXTENSION):
+            fileToSave = "%s.%s" % (fileToSave, FILE_EXTENSION)
 
-    savePresetToDict("last save")
+    savePresetToFile("last save")
 
     curfile = codecs.open(getVar("currentCeciliaFile", unicode=True), "r", encoding="utf-8")
     curtext = curfile.read()
     curfile.close()
-    delimiter = curtext.find(PRESETS_DELIMITER)
-    if delimiter != -1:
-        curtext = curtext[:delimiter]
 
     try:
         file = codecs.open(fileToSave, "w", encoding="utf-8")
     except IOError:
+        setVar("canGrabFocus", False)
         dlg = wx.MessageDialog(parent, 'Please verify permissions and write access on the file and try again.',
                             '"%s" could not be opened for writing' % (fileToSave), wx.OK | wx.ICON_EXCLAMATION)
         if dlg.ShowModal() == wx.ID_OK:
             dlg.Destroy()
+            setVar("canGrabFocus", True)
             return
 
     file.write(curtext.rstrip())
-    file.write("\n\n\n")
-    file.write(PRESETS_DELIMITER)
-    file.write("\n\n")
-    preset = pp.pformat(getVar("presets"), width=160)
-    preset = "CECILIA_PRESETS = " + preset
-    preset = ensureNFD(preset)
-    file.write(preset)
-
     file.close()
 
+    oldModuleName = os.path.split(getVar("currentCeciliaFile"))[1]
+    oldModuleName = os.path.splitext(oldModuleName)[0]
+    newModuleName = os.path.split(fileToSave)[1]
+    newModuleName = os.path.splitext(newModuleName)[0]
+    if not os.path.isdir(os.path.join(PRESETS_PATH, newModuleName)):
+        os.mkdir(os.path.join(PRESETS_PATH, newModuleName))
+    for f in os.listdir(os.path.join(PRESETS_PATH, oldModuleName)):
+        shutil.copyfile(os.path.join(PRESETS_PATH, oldModuleName, f), os.path.join(PRESETS_PATH, newModuleName, f))
+
     setVar("builtinModule", False)
+    setVar('currentModuleName', newModuleName)
     setVar("currentCeciliaFile", fileToSave)
-    setVar("isModified", False)
+    setVar("lastCeciliaFile", fileToSave)
+
+    if getVar("mainFrame") is not None:
+        getVar("mainFrame").newRecent(fileToSave)
 
     saveCompileBackupFile(fileToSave)
 
-    return True
-
 def openCeciliaFile(parent, openfile=None, builtin=False):
     if not openfile:
+        setVar("canGrabFocus", False)
         wildcard = "Cecilia file (*.%s)|*.%s" % (FILE_EXTENSION, FILE_EXTENSION)
         defaultPath = getVar("openFilePath", unicode=True)
         openDialog = wx.FileDialog(parent, message='Choose a Cecilia file to open...',
@@ -718,6 +717,7 @@ def openCeciliaFile(parent, openfile=None, builtin=False):
         else:
             cecFilePath = None
         openDialog.Destroy()
+        setVar("canGrabFocus", True)
 
         if cecFilePath is None:
             return
@@ -729,22 +729,29 @@ def openCeciliaFile(parent, openfile=None, builtin=False):
 
     snds = []
     if getVar("rememberedSound") and getVar("interfaceWidgets") and getVar("userInputs"):
-        names = [d['name'] for d in getVar("interfaceWidgets")]
-        keys = getVar("userInputs").keys()
-        sortlist = list(zip([names.index(k) for k in keys], keys))
-        sortlist.sort()
-        index, keys = list(zip(*sortlist))
-        for key in keys:
-            if getVar("userInputs")[key]['path'] != '':
-                snds.append(getVar("userInputs")[key]['path'])
+        try:
+            names = [d['name'] for d in getVar("interfaceWidgets")]
+            keys = getVar("userInputs").keys()
+            sortlist = list(zip([names.index(k) for k in keys], keys))
+            sortlist.sort()
+            index, keys = list(zip(*sortlist))
+            for key in keys:
+                if getVar("userInputs")[key]['path'] != '':
+                    snds.append(getVar("userInputs")[key]['path'])
+        except:
+            pass
 
-    if not closeCeciliaFile(parent):
-        return
+    if getVar("currentCeciliaFile"):
+        closeCeciliaFile(parent)
+
+    moduleName = os.path.split(cecFilePath)[1]
+    moduleName = os.path.splitext(moduleName)[0]
+    setVar('currentModuleName', moduleName)
+    if not os.path.isdir(os.path.join(PRESETS_PATH, moduleName)):
+        os.mkdir(os.path.join(PRESETS_PATH, moduleName))
 
     getVar("mainFrame").Hide()
 
-    setVar("isModified", False)
-
     if not getVar("audioServer").openCecFile(cecFilePath):
         return
 
@@ -752,8 +759,8 @@ def openCeciliaFile(parent, openfile=None, builtin=False):
     setVar("currentCeciliaFile", cecFilePath)
     setVar("lastCeciliaFile", cecFilePath)
 
-    if parent is not None:
-        parent.newRecent(cecFilePath)
+    if getVar("mainFrame") is not None:
+        getVar("mainFrame").newRecent(cecFilePath)
 
     saveCompileBackupFile(cecFilePath)
 
@@ -763,33 +770,37 @@ def openCeciliaFile(parent, openfile=None, builtin=False):
                 break
             cfilein.onLoadFile(snds[i])
 
-    savePresetToDict("init")
+    savePresetToFile("init")
 
-    if "last save" in getVar("presets"):
+    if os.path.isfile(os.path.join(PRESETS_PATH, moduleName, "last save")):
         setVar("presetToLoad", "last save")
 
     getVar("mainFrame").updateTitle()
 
-    wx.CallAfter(getVar("interface").Raise)
+    getVar("interface").Raise()
 
 def closeCeciliaFile(parent):
-    if not saveBeforeClose(parent):
-        return False
+    savePresetToFile("last save")
     getVar("mainFrame").closeInterface()
     setVar("currentCeciliaFile", '')
-    wx.CallLater(200, setVar, "isModified", False)
-    return True
 
 ###### Interface creation utilities ######
 def resetWidgetVariables():
     setVar("gainSlider", None)
+    setVar("plugins", [None] * NUM_OF_PLUGINS)
     setVar("userInputs", {})
+    for slider in getVar("userSliders"):
+        slider.cleanup()
     setVar("userSliders", [])
     setVar("userSamplers", [])
     setVar("userTogglePopups", [])
     setVar("samplerSliders", [])
-    setVar("grapher", None)
+    setVar("samplerTogglePopup", [])
+    getVar("presetPanel").cleanup()
+    getVar("presetPanel").parent = None
     setVar("presetPanel", None)
+    getVar("grapher").parent = None
+    setVar("grapher", None)
     setVar("tooltips", [])
 
 def parseInterfaceText():
@@ -802,6 +813,26 @@ def updateNchnlsDevices():
     except:
         pass
 
+def deepCopy(ori):
+    if type(ori) is dict:
+        new = {}
+        for key in ori:
+            new[key] = deepCopy(ori[key])
+        return new
+    elif type(ori) is list:
+        new = []
+        for data in ori:
+            new.append(deepCopy(data))
+        return new
+    elif type(ori) is tuple:
+        new = []
+        for data in ori:
+            new.append(deepCopy(data))
+        return new
+    else:
+        return ori
+    return new
+
 ###### Conversion functions #######
 def interpFloat(t, v1, v2):
     "interpolator for a single value; interprets t in [0-1] between v1 and v2"
@@ -887,3 +918,36 @@ def ensureNFD(unistr):
         return unistr
     else:
         return unicodedata.normalize(format, decstr)
+
+def checkForPresetsInCeciliaFile(filepath):
+    PRESETS_DELIMITER = "####################################\n" \
+                        "##### Cecilia reserved section #####\n" \
+                        "#### Presets saved from the app ####\n" \
+                        "####################################\n"
+
+    mylocals = {}
+    rewrite = False
+
+    with open(filepath, "r") as f:
+        text = f.read()
+        if PRESETS_DELIMITER in text:
+            rewrite = True
+            newtext = text[:text.find(PRESETS_DELIMITER) - 1]
+            text = text[text.find(PRESETS_DELIMITER) + len(PRESETS_DELIMITER) + 1:]
+            if text.strip() != "":
+                exec(text, globals(), mylocals)
+
+                presetsDir = os.path.join(PRESETS_PATH, getVar("currentModuleName")) 
+                if not os.path.isdir(presetsDir):
+                    os.mkdir(presetsDir)
+
+                for preset in mylocals["CECILIA_PRESETS"]:
+                    for key in list(mylocals["CECILIA_PRESETS"][preset]["plugins"]):
+                        mylocals["CECILIA_PRESETS"][preset]["plugins"][str(key)] = mylocals["CECILIA_PRESETS"][preset]["plugins"][key]
+                        del mylocals["CECILIA_PRESETS"][preset]["plugins"][key]
+                    msg = xmlrpclib.dumps((mylocals["CECILIA_PRESETS"][preset], ), allow_none=True)
+                    with open(os.path.join(presetsDir, preset), "w") as fw:
+                        fw.write(msg)
+    if rewrite:
+        with open(filepath, "w") as f:
+            f.write(newtext)
