@@ -1,6 +1,6 @@
-# encoding: utf-8
 """
-Copyright 2011 iACT, Universite de Montreal, Jean Piche, Olivier Belanger, Jean-Michel Dumas
+Copyright 2019 iACT, Universite de Montreal,
+Jean Piche, Olivier Belanger, Jean-Michel Dumas
 
 This file is part of Cecilia 5.
 
@@ -510,8 +510,15 @@ class CeciliaSlider:
 
     def setGraph(self, func):
         totalTime = CeciliaLib.getVar("totalTime")
-        func = [(int(x / totalTime * 8192), y) for x, y in func]
-        self.table.replace(func)
+        func2 = [(int(x / totalTime * 8192), y) for x, y in func[:]]
+
+        oldX = -1
+        for i, f in enumerate(func2):
+            if f[0] == oldX:
+                func2[i] = (f[0]+1, f[1])
+            oldX = func2[i][0]
+
+        self.table.replace(func2)
 
     def updateWidget(self):
         val = self.reader.get()
@@ -607,6 +614,13 @@ class CeciliaRange:
     def setGraph(self, which, func):
         totalTime = CeciliaLib.getVar("totalTime")
         func = [(int(x / totalTime * 8192), y) for x, y in func]
+
+        oldX = -1
+        for i, f in enumerate(func):
+            if f[0] == oldX:
+                func[i] = (f[0]+1, f[1])
+            oldX = func[i][0]
+
         if which == 0:
             self.table_min.replace(func)
         else:
@@ -679,6 +693,13 @@ class CeciliaSplitter:
     def setGraph(self, which, func):
         totalTime = CeciliaLib.getVar("totalTime")
         func = [(int(x / totalTime * 8192), y) for x, y in func]
+
+        oldX = -1
+        for i, f in enumerate(func):
+            if f[0] == oldX:
+                func[i] = (f[0]+1, f[1])
+            oldX = func[i][0]
+
         self.tables[which].replace(func)
 
     def updateWidget(self):
@@ -696,6 +717,13 @@ class CeciliaGraph:
                 break
 
         func = [(int(x / totalTime * (self.size - 1)), y) for x, y in line.getData()]
+
+        oldX = -1
+        for i, f in enumerate(func):
+            if f[0] == oldX:
+                func[i] = (f[0]+1, f[1])
+            oldX = func[i][0]
+
         curved = line.getCurved()
         if curved:
             self.table = CosTable(func, size=self.size)
@@ -713,6 +741,13 @@ class CeciliaGraph:
     def setValue(self, func):
         totalTime = CeciliaLib.getVar("totalTime")
         func = [(int(x / totalTime * (self.size - 1)), y) for x, y in func]
+
+        oldX = -1
+        for i, f in enumerate(func):
+            if f[0] == oldX:
+                func[i] = (f[0]+1, f[1])
+            oldX = func[i][0]
+
         self.table.replace(func)
 
 class BaseModule:
@@ -741,7 +776,7 @@ class BaseModule:
         self.polyphony_scaling = 1.0
         ###############################
 
-        interfaceWidgets = copy.deepcopy(CeciliaLib.getVar("interfaceWidgets"))
+        interfaceWidgets = CeciliaLib.getVar("interfaceWidgets")
         for widget in interfaceWidgets:
             if widget['type'] in ["cslider", "crange", "csplitter"]:
                 self._addSlider(widget, widget["type"])
@@ -851,7 +886,6 @@ class BaseModule:
         """
         Sets the Server's global seed used by objects from the random family.
 
-
         """
         CeciliaLib.getVar("audioServer").server.globalseed = x
     ############################
@@ -910,7 +944,7 @@ class BaseModule:
                                     tmpout = OscDataSend("f", widget.OSCOut[side][1], widget.OSCOut[side][2], widget.OSCOut[side][0])
                                     tmpout.send([val])
                                     self._OSCOutList.append(tmpout)
-                    else: # slider
+                    else: # slider TODO: could be merged with sampler slider code.
                         widget = slider.widget
                         path = widget.openSndCtrl[1]
                         val = rescale(widget.getValue(), xmin=widget.getMinValue(), xmax=widget.getMaxValue(), xlog=widget.getLog())
@@ -999,16 +1033,15 @@ class BaseModule:
         self._graphs[dic["name"]] = CeciliaGraph(dic)
         setattr(self, dic["name"], self._graphs[dic["name"]].sig())
 
-    def __del__(self):
+    def _cleanup(self):
         self.oscReceivers = {}
         self._OSCOutList = []
         for key in list(self.__dict__.keys()):
             del self.__dict__[key]
-        del self
 
 class CeciliaPlugin:
     def __init__(self, input, params=None, knobs=None):
-        self.input = InputFader(input)
+        self.input = Sig(input)
         gliss = 0.05
         totalTime = CeciliaLib.getVar("totalTime")
         if params is None:
@@ -1016,6 +1049,7 @@ class CeciliaPlugin:
             self._p2 = SigTo(0, time=0.025)
             self._p3 = SigTo(0, time=0.025)
         else:
+            # TODO: merge these three chunks in a single loop.
             self.widget_p1 = knobs[0]
             name = self.widget_p1.getName()
             for line in CeciliaLib.getVar("grapher").plotter.getData():
@@ -1038,7 +1072,7 @@ class CeciliaPlugin:
             self._p1 = SigTo(params[0], time=gliss, init=params[0])
             if self.rec_p1:
                 self.record_p1 = ControlRec(self._p1, filename=self.widget_p1.getPath(),
-                                        rate=1000, dur=totalTime).play()
+                                            rate=1000, dur=totalTime).play()
             if self.play_p1 > 0:
                 data = line.getData()
                 data = [tuple(x) for x in data]
@@ -1074,7 +1108,7 @@ class CeciliaPlugin:
             self._p2 = SigTo(params[1], time=gliss, init=params[1])
             if self.rec_p2:
                 self.record_p2 = ControlRec(self._p2, filename=self.widget_p2.getPath(),
-                                        rate=1000, dur=totalTime).play()
+                                            rate=1000, dur=totalTime).play()
             if self.play_p2 > 0:
                 data = line.getData()
                 data = [tuple(x) for x in data]
@@ -1152,8 +1186,8 @@ class CeciliaPlugin:
         else:
             self.out.interp = 1
 
-    def setInput(self, input, fadetime=0.05):
-        self.input.setInput(input, fadetime)
+    def setInput(self, input):
+        self.input.value = input
 
     def setGraph(self, which, func):
         totalTime = CeciliaLib.getVar("totalTime")
@@ -1480,7 +1514,8 @@ class AudioServer():
         jackname = CeciliaLib.getVar("jack").get("client", "cecilia5")
         if CeciliaLib.getVar("DEBUG"):
             print("AUDIO CONFIG:")
-            print("sr: %s, buffer size: %s, num of channels: %s, duplex: %s, host: %s, output device: %s, input device: %s" % (sr, bufsize, nchnls, duplex, host, outdev, indev))
+            template = "sr: %s, buffer size: %s, num of channels: %s, duplex: %s, host: %s, output device: %s, input device: %s"
+            print(template % (sr, bufsize, nchnls, duplex, host, outdev, indev))
             print("first physical input: %s, first physical output: %s\n" % (firstin, firstout))
         self.server = Server(sr=sr, buffersize=bufsize, nchnls=nchnls, duplex=duplex, audio=host, jackname=jackname)
         if CeciliaLib.getVar("DEBUG"):
@@ -1587,7 +1622,8 @@ class AudioServer():
         sr, bufsize, nchnls, duplex, host, outdev, indev, firstin, firstout = self.getPrefs()
         if CeciliaLib.getVar("DEBUG"):
             print("AUDIO CONFIG:")
-            print("sr: %s, buffer size: %s, num of channels: %s, duplex: %s, host: %s, output device: %s, input device: %s" % (sr, bufsize, nchnls, duplex, host, outdev, indev))
+            template = "sr: %s, buffer size: %s, num of channels: %s, duplex: %s, host: %s, output device: %s, input device: %s"
+            print(template % (sr, bufsize, nchnls, duplex, host, outdev, indev))
             print("first physical input: %s, first physical output: %s\n" % (firstin, firstout))
             print("MIDI CONFIG: \ninput device: %d\n" % CeciliaLib.getVar("midiDeviceIn"))
         self.server.setSamplingRate(sr)
@@ -1667,19 +1703,13 @@ class AudioServer():
         self.server.setSamplingRate(sr)
 
     def recordOptions(self, dur, filename, fileformat, sampletype):
-        self.server.recordOptions(dur=dur, filename=filename, fileformat=fileformat, sampletype=sampletype)
+        self.server.recordOptions(dur, filename, fileformat, sampletype)
 
     def isAudioServerRunning(self):
-        if self.server.getIsStarted():
-            return True
-        else:
-            return False
+        return self.server.getIsStarted()
 
     def isAudioServerBooted(self):
-        if self.server.getIsBooted():
-            return True
-        else:
-            return False
+        return self.server.getIsBooted()
 
     def compileRuntimeError(self, filepath, title, msg):
         error = traceback.format_exc()
@@ -1719,22 +1749,25 @@ class AudioServer():
         CeciliaLib.showErrorDialog(title, msg)
 
     def openCecFile(self, filepath):
+        global Module, Interface
+
+        try:
+            del Module, Interface
+        except:
+            pass
+
+        if not serverBooted():
+            self.boot()
+
+        self.cleanup_module()
+
         CeciliaLib.setVar("currentModule", None)
         CeciliaLib.setVar("currentModuleRef", None)
         CeciliaLib.setVar("interfaceWidgets", [])
         CeciliaLib.setVar("startOffset", 0.0)
-        try:
-            global Module, Interface
-            del Module, Interface
-        except:
-            pass
-        try:
-            global CECILIA_PRESETS
-            del CECILIA_PRESETS
-        except:
-            pass
-        if not serverBooted():
-            self.boot()
+
+        CeciliaLib.checkForPresetsInCeciliaFile(filepath)
+
         try:
             with open(filepath, "r") as f:
                 exec(f.read(), globals())
@@ -1750,17 +1783,13 @@ class AudioServer():
             return False
 
         CeciliaLib.setVar("currentModuleRef", copy.deepcopy(Module))
-        CeciliaLib.setVar("interfaceWidgets", copy.deepcopy(Interface))
+        CeciliaLib.setVar("interfaceWidgets", CeciliaLib.deepCopy(Interface))
 
-        try:
-            CeciliaLib.setVar("presets", copy.deepcopy(CECILIA_PRESETS))
-        except:
-            CeciliaLib.setVar("presets", {})
         CeciliaLib.getVar("mainFrame").onUpdateInterface(None)
 
         return True
 
-    def loadModule(self, module):
+    def cleanup_module(self):
         for i in range(NUM_OF_PLUGINS):
             if self.pluginObjs[i] is not None:
                del self.pluginObjs[i].out
@@ -1774,32 +1803,34 @@ class AudioServer():
             if self.out is not None:
                 del self.out
                 self.out = None
-        if CeciliaLib.getVar("systemPlatform") == "darwin":
-            try:
-                del self.globalamp
-            except:
-                pass
-            try:
-                del self.endcall
-            except:
-                pass
-            try:
-                del self.recorder
-                del self.recamp
-            except:
-                pass
-            try:
-                del self.checkCtl7
-                del self.onNewCtl7Value
-                del self.ctl7TrigFunc
-            except:
-                pass
         try:
-            CeciliaLib.getVar("currentModule").__del__()
+            del self.globalamp
+        except:
+            pass
+        try:
+            del self.endcall
+        except:
+            pass
+        try:
+            del self.recorder
+            del self.recamp
+        except:
+            pass
+        try:
+            del self.checkCtl7
+            del self.onNewCtl7Value
+            del self.ctl7TrigFunc
+        except:
+            pass
+        try:
+            CeciliaLib.getVar("currentModule")._cleanup()
             CeciliaLib.setVar("currentModule", None)
         except:
             pass
 
+    def loadModule(self, module):
+        self.cleanup_module()
+
         try:
             currentModule = module()
         except Exception as e:
@@ -1961,10 +1992,7 @@ class AudioServer():
                 defaultOutputDriver, midiDriverList, midiDriverIndexes, defaultMidiDriver
 
     def validateAudioFile(self, path):
-        if sndinfo(path) is not None:
-            return True
-        else:
-            return False
+        return sndinfo(path) is not None
 
     def getSoundInfo(self, path):
         """
@@ -2001,7 +2029,8 @@ class AudioServer():
 
             return (chnls, samprate, dur, tableFrac, nsamps, bitrate, format)
         else:
-            print('Unable to get sound infos. "%s" bypassed!' % path)
+            if CeciliaLib.getVar("DEBUG"):
+                print('Unable to get sound infos. "%s" bypassed!' % path)
             return None
 
     def getSoundsFromList(self, pathList):
@@ -2023,4 +2052,4 @@ class AudioServer():
                     print('not a file')
         if CeciliaLib.getVar("DEBUG"):
             print()
-        return soundDict
\ No newline at end of file
+        return soundDict
