@@ -1,6 +1,6 @@
-# encoding: utf-8
 """
-Copyright 2011 iACT, Universite de Montreal, Jean Piche, Olivier Belanger, Jean-Michel Dumas
+Copyright 2011 iACT, Universite de Montreal,
+Jean Piche, Olivier Belanger, Jean-Michel Dumas
 
 This file is part of Cecilia 5.
 
@@ -23,13 +23,9 @@ from .images import *
 BUILD_RST = False
 
 APP_NAME = 'Cecilia5'
-APP_VERSION = '5.3.5'
-APP_COPYRIGHT = 'iACT,  2017'
+APP_VERSION = '5.4.0'
+APP_COPYRIGHT = 'iACT,  2019'
 FILE_EXTENSION = "c5"
-PRESETS_DELIMITER = "####################################\n" \
-                    "##### Cecilia reserved section #####\n" \
-                    "#### Presets saved from the app ####\n" \
-                    "####################################\n"
 
 if sys.platform == "win32":
     FILE_ENCODING = "mbcs"
@@ -48,109 +44,127 @@ if not os.path.isdir(RESOURCES_PATH) and sys.platform == "win32":
     RESOURCES_PATH = os.path.join(os.getenv("ProgramFiles"), "Cecilia5", "Resources")
 
 TMP_PATH = os.path.join(os.path.expanduser('~'), '.cecilia5')
-PREFERENCES_FILE = os.path.join(TMP_PATH, 'ceciliaPrefs.txt')
-DOC_PATH = os.path.join(TMP_PATH, 'doc')
-MODULES_PATH = os.path.join(RESOURCES_PATH, 'modules')
+if not os.path.isdir(TMP_PATH):
+    os.mkdir(TMP_PATH)
+
+RECENT_FILE_PATH = os.path.join(TMP_PATH, '.recent.txt')
+if not os.path.isfile(RECENT_FILE_PATH):
+    f = open(RECENT_FILE_PATH, "w")
+    f.close()
+
+PRESETS_PATH = os.path.join(TMP_PATH, 'presets')
+if not os.path.isdir(PRESETS_PATH):
+    os.mkdir(PRESETS_PATH)
+
 AUTOMATION_SAVE_PATH = os.path.join(TMP_PATH, 'automation_save')
+if not os.path.isdir(AUTOMATION_SAVE_PATH):
+    os.mkdir(AUTOMATION_SAVE_PATH)
+
+DOC_PATH = os.path.join(TMP_PATH, 'doc')
+if not os.path.isdir(DOC_PATH):
+    os.mkdir(DOC_PATH)
+
+PREFERENCES_FILE = os.path.join(TMP_PATH, 'ceciliaPrefs.txt')
+MODULES_PATH = os.path.join(RESOURCES_PATH, 'modules')
 SPLASH_FILE_PATH = os.path.join(RESOURCES_PATH, "Cecilia_splash.png")
 MODULE_COMPILE_BACKUP_PATH = os.path.join(TMP_PATH, 'moduleCompileBackup.c5')
 MODULE_RUNTIME_BACKUP_PATH = os.path.join(TMP_PATH, 'moduleRuntimeBackup.c5')
 
 # Meter icons
-ICON_VUMETER = catalog['vu-metre2.png']
-ICON_VUMETER_DARK = catalog['vu-metre-dark2.png']
+ICON_VUMETER = catalog['vu-metre2']
+ICON_VUMETER_DARK = catalog['vu-metre-dark2']
 # Plugin icons
-ICON_PLUGINS_KNOB = catalog['knob-trans-sm.png']
-ICON_PLUGINS_KNOB_DISABLE = catalog['knob-disab-sm.png']
-ICON_PLUGINS_ARROW_UP = catalog['arrow_up.png']
-ICON_PLUGINS_ARROW_UP_HOVER = catalog['arrow_up_hover.png']
-ICON_PLUGINS_ARROW_DOWN = catalog['arrow_down.png']
-ICON_PLUGINS_ARROW_DOWN_HOVER = catalog['arrow_down_hover.png']
+ICON_PLUGINS_KNOB = catalog['knob-trans-sm']
+ICON_PLUGINS_KNOB_DISABLE = catalog['knob-disab-sm']
+ICON_PLUGINS_ARROW_UP = catalog['arrow_up']
+ICON_PLUGINS_ARROW_UP_HOVER = catalog['arrow_up_hover']
+ICON_PLUGINS_ARROW_DOWN = catalog['arrow_down']
+ICON_PLUGINS_ARROW_DOWN_HOVER = catalog['arrow_down_hover']
 # Toolbox icons
-ICON_TB_LOAD = catalog['load-normal-trans.png']
-ICON_TB_LOAD_OVER = catalog['load-hover-trans.png']
-ICON_TB_SAVE = catalog['save-normal-trans.png']
-ICON_TB_SAVE_OVER = catalog['save-hover-trans.png']
-ICON_TB_RESET = catalog['reset-normal-trans.png']
-ICON_TB_RESET_OVER = catalog['reset-hover-trans.png']
-ICON_TB_SHOW = catalog['show-normal-trans.png']
-ICON_TB_SHOW_OVER = catalog['show-hover-trans.png']
-ICON_TB_HIDE = catalog['hide-normal-trans.png']
-ICON_TB_HIDE_OVER = catalog['hide-hover-trans.png']
-ICON_TB_RECYCLE = catalog['recycle-normal-trans.png']
-ICON_TB_RECYCLE_OVER = catalog['recycle-hover-trans.png']
-ICON_TB_PLAY = catalog['play-normal-trans.png']
-ICON_TB_PLAY_OVER = catalog['play-hover-trans.png']
-ICON_TB_EDIT = catalog['edit-normal-trans.png']
-ICON_TB_EDIT_OVER = catalog['edit-hover-trans.png']
-ICON_TB_OPEN = catalog['open-normal-trans.png']
-ICON_TB_OPEN_OVER = catalog['open-hover-trans.png']
-ICON_TB_CLOSE = catalog['close-normal-trans.png']
-ICON_TB_CLOSE_OVER = catalog['close-hover-trans.png']
-ICON_TB_TIME = catalog['time-normal-trans.png']
-ICON_TB_TIME_OVER = catalog['time-hover-trans.png']
-ICON_TB_DELETE = catalog['delete-normal-trans.png']
-ICON_TB_DELETE_OVER = catalog['delete-hover-trans.png']
+ICON_TB_LOAD = catalog['load-normal-trans']
+ICON_TB_LOAD_OVER = catalog['load-hover-trans']
+ICON_TB_SAVE = catalog['save-normal-trans']
+ICON_TB_SAVE_OVER = catalog['save-hover-trans']
+ICON_TB_RESET = catalog['reset-normal-trans']
+ICON_TB_RESET_OVER = catalog['reset-hover-trans']
+ICON_TB_SHOW = catalog['show-normal-trans']
+ICON_TB_SHOW_OVER = catalog['show-hover-trans']
+ICON_TB_HIDE = catalog['hide-normal-trans']
+ICON_TB_HIDE_OVER = catalog['hide-hover-trans']
+ICON_TB_RECYCLE = catalog['recycle-normal-trans']
+ICON_TB_RECYCLE_OVER = catalog['recycle-hover-trans']
+ICON_TB_PLAY = catalog['play-normal-trans']
+ICON_TB_PLAY_OVER = catalog['play-hover-trans']
+ICON_TB_EDIT = catalog['edit-normal-trans']
+ICON_TB_EDIT_OVER = catalog['edit-hover-trans']
+ICON_TB_OPEN = catalog['open-normal-trans']
+ICON_TB_OPEN_OVER = catalog['open-hover-trans']
+ICON_TB_CLOSE = catalog['close-normal-trans']
+ICON_TB_CLOSE_OVER = catalog['close-hover-trans']
+ICON_TB_TIME = catalog['time-normal-trans']
+ICON_TB_TIME_OVER = catalog['time-hover-trans']
+ICON_TB_DELETE = catalog['delete-normal-trans']
+ICON_TB_DELETE_OVER = catalog['delete-hover-trans']
 # RadioToolbox icons
-ICON_RTB_POINTER = catalog['pointer-normal-trans.png']
-ICON_RTB_POINTER_OVER = catalog['pointer-hover-trans.png']
-ICON_RTB_POINTER_CLICK = catalog['pointer-click-trans.png']
-ICON_RTB_PENCIL = catalog['pencil-normal-trans.png']
-ICON_RTB_PENCIL_OVER = catalog['pencil-hover-trans.png']
-ICON_RTB_PENCIL_CLICK = catalog['pencil-click-trans.png']
-ICON_RTB_ZOOM = catalog['zoom-normal-trans.png']
-ICON_RTB_ZOOM_OVER = catalog['zoom-hover-trans.png']
-ICON_RTB_ZOOM_CLICK = catalog['zoom-click-trans.png']
-ICON_RTB_HAND = catalog['hand-normal-trans.png']
-ICON_RTB_HAND_OVER = catalog['hand-hover-trans.png']
-ICON_RTB_HAND_CLICK = catalog['hand-click-trans.png']
+ICON_RTB_POINTER = catalog['pointer-normal-trans']
+ICON_RTB_POINTER_OVER = catalog['pointer-hover-trans']
+ICON_RTB_POINTER_CLICK = catalog['pointer-click-trans']
+ICON_RTB_PENCIL = catalog['pencil-normal-trans']
+ICON_RTB_PENCIL_OVER = catalog['pencil-hover-trans']
+ICON_RTB_PENCIL_CLICK = catalog['pencil-click-trans']
+ICON_RTB_ZOOM = catalog['zoom-normal-trans']
+ICON_RTB_ZOOM_OVER = catalog['zoom-hover-trans']
+ICON_RTB_ZOOM_CLICK = catalog['zoom-click-trans']
+ICON_RTB_HAND = catalog['hand-normal-trans']
+ICON_RTB_HAND_OVER = catalog['hand-hover-trans']
+ICON_RTB_HAND_CLICK = catalog['hand-click-trans']
 # PrefRadioToolbox icons
-ICON_PREF_AUDIO = catalog['audio-normal-trans.png']
-ICON_PREF_AUDIO_OVER = catalog['audio-hover-trans.png']
-ICON_PREF_AUDIO_CLICK = catalog['audio-click-trans.png']
-ICON_PREF_CECILIA = catalog['cecilia-normal-trans.png']
-ICON_PREF_CECILIA_OVER = catalog['cecilia-hover-trans.png']
-ICON_PREF_CECILIA_CLICK = catalog['cecilia-click-trans.png']
-ICON_PREF_FILER = catalog['filer-normal-trans.png']
-ICON_PREF_FILER_OVER = catalog['filer-hover-trans.png']
-ICON_PREF_FILER_CLICK = catalog['filer-click-trans.png']
-ICON_PREF_PATH = catalog['path-normal-trans.png']
-ICON_PREF_PATH_OVER = catalog['path-hover-trans.png']
-ICON_PREF_PATH_CLICK = catalog['path-click-trans.png']
-ICON_PREF_MIDI = catalog['midi-normal-trans.png']
-ICON_PREF_MIDI_OVER = catalog['midi-hover-trans.png']
-ICON_PREF_MIDI_CLICK = catalog['midi-click-trans.png']
+ICON_PREF_AUDIO = catalog['audio-normal-trans']
+ICON_PREF_AUDIO_OVER = catalog['audio-hover-trans']
+ICON_PREF_AUDIO_CLICK = catalog['audio-click-trans']
+ICON_PREF_CECILIA = catalog['cecilia-normal-trans']
+ICON_PREF_CECILIA_OVER = catalog['cecilia-hover-trans']
+ICON_PREF_CECILIA_CLICK = catalog['cecilia-click-trans']
+ICON_PREF_FILER = catalog['filer-normal-trans']
+ICON_PREF_FILER_OVER = catalog['filer-hover-trans']
+ICON_PREF_FILER_CLICK = catalog['filer-click-trans']
+ICON_PREF_PATH = catalog['path-normal-trans']
+ICON_PREF_PATH_OVER = catalog['path-hover-trans']
+ICON_PREF_PATH_CLICK = catalog['path-click-trans']
+ICON_PREF_MIDI = catalog['midi-normal-trans']
+ICON_PREF_MIDI_OVER = catalog['midi-hover-trans']
+ICON_PREF_MIDI_CLICK = catalog['midi-click-trans']
 # PaletteToolBox icons
-ICON_PTB_PROCESS = catalog['process-normal-trans.png']
-ICON_PTB_PROCESS_OVER = catalog['process-hover-trans.png']
-ICON_PTB_RANDOM = catalog['random-normal-trans.png']
-ICON_PTB_RANDOM_OVER = catalog['random-hover-trans.png']
-ICON_PTB_WAVES = catalog['waves-normal-trans.png']
-ICON_PTB_WAVES_OVER = catalog['waves-hover-trans.png']
+ICON_PTB_PROCESS = catalog['process-normal-trans']
+ICON_PTB_PROCESS_OVER = catalog['process-hover-trans']
+ICON_PTB_RANDOM = catalog['random-normal-trans']
+ICON_PTB_RANDOM_OVER = catalog['random-hover-trans']
+ICON_PTB_WAVES = catalog['waves-normal-trans']
+ICON_PTB_WAVES_OVER = catalog['waves-hover-trans']
 # Input icons
-ICON_INPUT_1_FILE = catalog['input-1-file.png']
-ICON_INPUT_2_LIVE = catalog['input-2-live.png']
-ICON_INPUT_3_MIC = catalog['input-3-mic.png']
-ICON_INPUT_4_MIC_RECIRC = catalog['input-4-mic-recirc.png']
+ICON_INPUT_1_FILE = catalog['input-1-file']
+ICON_INPUT_2_LIVE = catalog['input-2-live']
+ICON_INPUT_3_MIC = catalog['input-3-mic']
+ICON_INPUT_4_MIC_RECIRC = catalog['input-4-mic-recirc']
 # Crossfade icons
-ICON_XFADE_LINEAR = catalog['xfade-linear.png']
-ICON_XFADE_POWER = catalog['xfade-power.png']
-ICON_XFADE_SIGMOID = catalog['xfade-sigmoid.png']
+ICON_XFADE_LINEAR = catalog['xfade-linear']
+ICON_XFADE_POWER = catalog['xfade-power']
+ICON_XFADE_SIGMOID = catalog['xfade-sigmoid']
 # Mario bros
-ICON_MARIO1 = catalog['Mario1.png']
-ICON_MARIO2 = catalog['Mario2.png']
-ICON_MARIO3 = catalog['Mario3.png']
-ICON_MARIO4 = catalog['Mario4.png']
-ICON_MARIO5 = catalog['Mario5.png']
-ICON_MARIO6 = catalog['Mario6.png']
+ICON_MARIO1 = catalog['Mario1']
+ICON_MARIO2 = catalog['Mario2']
+ICON_MARIO3 = catalog['Mario3']
+ICON_MARIO4 = catalog['Mario4']
+ICON_MARIO5 = catalog['Mario5']
+ICON_MARIO6 = catalog['Mario6']
 # Grapher background
-ICON_GRAPHER_BACKGROUND = catalog['Grapher_background.png']
+ICON_GRAPHER_BACKGROUND = catalog['Grapher_background']
 # About icon
-ICON_CECILIA_ABOUT_SMALL = catalog['Cecilia_about_small.png']
+ICON_CECILIA_ABOUT_SMALL = catalog['Cecilia_about_small']
 # Doc frame icons
-ICON_DOC_PREVIOUS = catalog['previous_24.png']
-ICON_DOC_NEXT = catalog['next_24.png']
-ICON_DOC_UP = catalog['up_24.png']
+ICON_DOC_PREVIOUS = catalog['previous_24']
+ICON_DOC_NEXT = catalog['next_24']
+ICON_DOC_UP = catalog['up_24']
 
 # Audio drivers
 if sys.platform == 'darwin' and '/%s.app' % APP_NAME in os.getcwd():
@@ -162,7 +176,7 @@ elif sys.platform == 'win32':
 else:
     AUDIO_DRIVERS = ['portaudio', 'jack']
 
-# MIDI drivers
+# MIDI drivers (TODO: add jackmidi on linux)
 MIDI_DRIVERS = ['portmidi']
 
 # plugin types
@@ -249,27 +263,27 @@ else:
     LIST_ENTRY_FONT = 12
 
 # Colours
-BACKGROUND_COLOUR = "#666666"
-GRAPHER_BACK_COLOUR = "#D1D1D1"
-TITLE_BACK_COLOUR = "#333333"
+BACKGROUND_COLOUR = "#494949"
+GRAPHER_BACK_COLOUR = "#C1C1C1"
+TITLE_BACK_COLOUR = "#282828"
 SECTION_TITLE_COLOUR = "#CCCCCC"
 WHITE_COLOUR = "#FFFFFF"
 LIGHTGREY_COLOUR = "#999999"
 GREY_COLOUR = "#666666"
 BLACK_COLOUR = "#000000"
 BORDER_COLOUR = "#444444"
-CLOSEBOX_INSIDE_COLOUR = "#8896BB"
+BUTTON_BACK_COLOUR = "#506077" #"#8896BB"
 TEXT_LABELFORWIDGET_COLOUR = "#FFFFFF"
 GRADIENT_DARK_COLOUR = "#313740"
 WIDGET_BORDER_COLOUR = "#BBBBBB"
 KNOB_BORDER_COLOUR = "#929292"
-POPUP_BACK_COLOUR = "#80A0B0"
+POPUP_BACK_COLOUR = "#6F7F97" #"#80A0B0"
 POPUP_DISABLE_COLOUR = "#888888"
 POPUP_LABEL_COLOUR = "#FFFFFF"
 POPUP_DISABLE_LABEL_COLOUR = "#333333"
 LABEL_LABEL_COLOUR = "#FFFFFF"
 LABEL_BACK_COLOUR = "#666666"
-CPOLY_COLOUR = "#555555"
+CPOLY_COLOUR = "#444444"
 TOGGLE_LABEL_COLOUR = "#FFFFFF"
 ENTRYUNIT_HIGHLIGHT_COLOUR = "#222222"
 ENTRYUNIT_BACK_COLOUR = "#666666"
@@ -288,55 +302,43 @@ CONTROLSLIDER_SELECTED_COLOUR = '#333333'
 CONTROLSLIDER_TEXT_COLOUR = '#FFFFFF'
 CONTROLLABEL_BACK_COLOUR = "#6F7F97"
 PLUGINPOPUP_BACK_COLOUR = "#506077"
-TR_BACK_COLOUR = "#6F7F97"
+TR_BACK_COLOUR = "#506077" #"#6F7F97"
 TR_BORDER_COLOUR = '#BBBBBB'
 TR_PLAY_NORMAL_COLOUR = '#009911'
 TR_PLAY_CLICK_COLOUR = '#007A29'
 TR_RECORD_OFF_COLOUR = '#6E3131'
 TR_RECORD_ON_COLOUR = '#FF0000'
-PREFS_FOREGROUND = '#222222'
+TR_CLOCKER_BACK_COLOUR = "#506077" #'#99A7CC'
+PREFS_FOREGROUND = '#EEEEEE'
 PREFS_PATH_BACKGROUND = '#AAAAAA'
 
 # Hue, Brightness, Saturation
-COLOUR_CLASSES = {'green': [100., 0.25, .75],
-        'forestgreen': [85., 0.3, .6],
-        'olivegreen': [75., 0.4, .6],
-        'lightgreen': [65., 0.45, .4],
-        'blue': [230., 0.35, .55],
-        'marineblue': [220., 0.4, .45],
-        'royalblue': [203., 0.43, .4],
-        'lightblue': [190., 0.5, .35],
-        'red': [0., .28, .75],
-        'brightred': [355., 0.35, .65],
-        'brightblue': [350., 0.45, .55],
-        'brightgreen': [345., 0.5, .45],
-        'orange': [20., 0.35, .85],
-        'khaki': [18., 0.4, .7],
-        'tan': [16., 0.47, .55],
-        'chorusyellow': [14., 0.55, .4],
-        'filterred': [342., 0.44, .47],
-        'compblue': [240., 0.44, .22],
-        'grey': [0., 0.34, 0.],
-        'green1': [100., 0.25, .75], # filters popup and freq
-        'green2': [85., 0.3, .6], # filters Q
-        'green3': [75., 0.4, .6],
-        'green4': [65., 0.45, .4],
-        'blue1': [230., 0.35, .55], # dry/wet, amplitude, balance
-        'blue2': [220., 0.4, .45],
-        'blue3': [203., 0.43, .4],
-        'blue4': [190., 0.5, .35],
-        'red1': [0., .28, .75], # Pitch, transposition
-        'red2': [355., 0.35, .65],
-        'red3': [350., 0.45, .55],
-        'red4': [345., 0.5, .45],
-        'orange1': [22., 0.35, .85], # post-processing knob lines
-        'orange2': [18., 0.4, .7],
-        'orange3': [16., 0.47, .55],
-        'orange4': [14., 0.52, .45],
-        'purple1': [290., 0.3, .65], # process specific parameters
-        'purple2': [280., 0.4, .6],
-        'purple3': [270., 0.47, .55],
-        'purple4': [260., 0.52, .45]
+COLOUR_CLASSES = {  'grey': [0., 0.25, 0.],
+                    'green': [100., 0.25, .75],
+                    'green1': [100., 0.25, .75], # filters popup and freq
+                    'green2': [85., 0.3, .6], # filters Q
+                    'green3': [75., 0.4, .6],
+                    'green4': [65., 0.45, .4],
+                    'blue': [230., 0.35, .55],
+                    'blue1': [230., 0.35, .55], # dry/wet, amplitude, balance
+                    'blue2': [220., 0.4, .45],
+                    'blue3': [203., 0.43, .4],
+                    'blue4': [190., 0.5, .35],
+                    'red': [0., .28, .75],
+                    'red1': [0., .28, .75], # Pitch, transposition
+                    'red2': [355., 0.35, .65],
+                    'red3': [350., 0.45, .55],
+                    'red4': [345., 0.5, .45],
+                    'orange': [20., 0.35, .85],
+                    'orange1': [22., 0.35, .85], # post-processing knob lines
+                    'orange2': [18., 0.4, .7],
+                    'orange3': [16., 0.47, .55],
+                    'orange4': [14., 0.52, .45],
+                    'purple': [290., 0.3, .65],
+                    'purple1': [290., 0.3, .65], # process specific parameters
+                    'purple2': [280., 0.4, .6],
+                    'purple3': [270., 0.47, .55],
+                    'purple4': [260., 0.52, .45]
 }
 
 # ToolTips
@@ -691,11 +693,12 @@ TT_GRAPHER = """GRAPHER BINDINGS
 
 Pointer tool:
   - Click and drag line to move it horizontally.
-  - Double-click on line to toggle between curved and straight segments.
+  - Shift+Click on line to add a point.
+  - Double-click on line to toggle curved mode.
   - Click on point or drag to select points.
   - Click and drag to move point or selected points.
-  - Holding Alt key when dragging clip the horizontal position.
-  - Holding Shift+Alt key when dragging clip the vertical position.
+  - Dragging with Alt key clips horizontal position.
+  - Dragging with Shift+Alt clips vertical position.
   - Double-click anywhere to add point.
   - Delete key to delete selected points.
 
@@ -708,7 +711,7 @@ Zoom tool:
     - Escape key to reset zoom level.
 
 Hand tool:
-    - When zoomed, click and drag to move view of the grapher."""
+    - When zoomed, click and drag to move the view."""
 
 TT_POPUP = """POPUP
 Choose amongst a predefined list of elements.
