@@ -1,15 +1,17 @@
-import os, sys
+import os
 
-version = sys.version_info[:2]
-
-os.system('C:\Python%d%d\Scripts\pyi-makespec -F -c --icon=Resources\Cecilia5.ico "Cecilia5.py"' % version)
-os.system('C:\Python%d%d\Scripts\pyi-build "Cecilia5.spec"' % version)
+flags = "--clean -F -c"
+hidden = "--hidden-import wx.adv --hidden-import wx.html --hidden-import wx.xml"
+icon = "--icon=Resources\Cecilia5.ico"
+os.system('pyinstaller %s %s %s "Cecilia5.py"' % (flags, hidden, icon))
 
 os.system("git checkout-index -a -f --prefix=Cecilia5_Win/")
 os.system("copy dist\Cecilia5.exe Cecilia5_Win /Y")
 os.system("rmdir /Q /S Cecilia5_Win\scripts")
 os.system("rmdir /Q /S Cecilia5_Win\doc-en")
 os.remove("Cecilia5_Win/Cecilia5.py")
+os.remove("Cecilia5_Win/.gitignore")
+os.remove("Cecilia5_Win/setup.py")
 os.remove("Cecilia5_Win/Resources/Cecilia5.icns")
 os.remove("Cecilia5_Win/Resources/CeciliaFileIcon5.icns")
 os.remove("Cecilia5.spec")
